Package: perpetual
Type: Package
Title: PerpetualBooster
Version: 3.0.0
Authors@R: c(
    person("Mutlu", "Simsek", email = "msimsek@perpetual-ml.com", role = c("aut", "cre")),
    person("Serkan", "Korkmaz", email = "serkor1@duck.com", role = "aut"),
    person("Pieter", "Pel", email = "pelpieter@gmail.com", role = "aut")
  )
Description: A self-generalizing gradient boosting machine that doesn't need hyperparameter optimization.
URL: https://perpetual-ml.github.io/perpetual/r/
License: Apache License (== 2.0)
Suggests: 
    jsonlite,
    testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
Biarch: true
SystemRequirements: Cargo (Rust's package manager), rustc
RoxygenNote: 7.3.3
